@@ -1,5 +1,5 @@
 ;;;; psgml-dtd.el --- DTD parser for SGML-editing mode with parsing support
-;; $Id: psgml-dtd.el,v 1.2 1998/02/10 16:18:38 steveb Exp $
+;; $Id: psgml-dtd.el,v 1.3 2000/03/27 14:10:31 andreasj Exp $
 
 ;; Copyright (C) 1994 Lennart Staflin
 
@@ -281,10 +281,10 @@ Syntax: var dfa-expr &body forms"
       (prog1 (if (sgml-is-delim "NULL" digit)
 		 (string-to-int (sgml-check-nametoken))
 	       (let ((spec (sgml-check-name)))
-		 (or (cdr (assoc spec '(("re" . 10)
-					("rs" . 1)
-					("tab" . 9)
-					("space" . 32))))
+		 (or (cdr (assoc spec '(("RE" . 10)
+					("RS" . 1)
+					("TAB" . 9)
+					("SPACE" . 32))))
 		     ;; *** What to do with other names?
 		     127)))
 	(or (sgml-parse-delim "REFC")
@@ -409,8 +409,8 @@ Case transformed for general names."
 	      name))))))
 
 
-(defun sgml-check-external ()
-  (or (sgml-parse-external)
+(defun sgml-check-external (&optional pubid-ok)
+  (or (sgml-parse-external pubid-ok)
       (sgml-parse-error "Expecting a PUBLIC or SYSTEM")))
 
 ;;;; Parse doctype: notation
@@ -425,7 +425,7 @@ Case transformed for general names."
   (sgml-skip-ps)
   (sgml-check-name)
   (sgml-skip-ps)
-  (sgml-check-external))
+  (sgml-check-external t))
 
 
 ;;;; Parse doctype: Element
@@ -443,11 +443,11 @@ Case transformed for general names."
 	   nil))))
 
 (defun sgml-parse-modifier ()
-  (cond ((sgml-parse-delim PLUS)
+  (cond ((sgml-parse-delim "PLUS")
 	 (function sgml-make-+))
-	((sgml-parse-delim REP)
+	((sgml-parse-delim "REP")
 	 (function sgml-make-*))
-	((sgml-parse-delim OPT)
+	((sgml-parse-delim "OPT")
 	 (function sgml-make-opt))))
 
 (defun sgml-check-primitive-content-token ()
@@ -474,7 +474,7 @@ Case transformed for general names."
 	(setq el (if con1
 		     (funcall con1 subs)
 		   (car subs)))))
-     ((sgml-parse-rni "pcdata")		; #PCDATA
+     ((sgml-parse-rni "PCDATA")         ; #PCDATA (FIXME: when changing case)
       (setq sgml-used-pcdata t)
       (setq el (sgml-make-pcdata)))
      ((sgml-parse-delim "DTGO")			; data tag group
@@ -521,7 +521,7 @@ Case transformed for general names."
 	 (sgml-check-content-model))
 	(t
 	 ;; ANY, CDATA, RCDATA or EMPTY
-	 (let ((dc (intern (upcase (sgml-check-name))))) 
+	 (let ((dc (intern (sgml-check-name)))) 
 	   (cond ((eq dc 'ANY)
 		  (setq sgml-used-pcdata t))
 		 ((eq dc 'CDATA)
@@ -529,7 +529,13 @@ Case transformed for general names."
 		    (sgml-error "XML forbids CDATA declared content.")))
 		 ((eq dc 'RCDATA)
 		  (when sgml-xml-p
-		    (sgml-error "XML forbids RCDATA declared content"))))
+		    (sgml-error "XML forbids RCDATA declared content")))
+                 ((eq dc 'EMPTY))
+                 (t
+                  (sgml-error "Exptected content model group or one of %s"
+                              (if sgml-xml-p
+                                  "ANY or EMPTY"
+                                  "ANY, CDATA, RCDATA or EMPTY"))))
 	   dc))))
 
 (defun sgml-parse-exeption (type)
@@ -578,6 +584,7 @@ Case transformed for general names."
   (let (name				; Name of entity
 	dest				; Entity table
 	(type 'text)			; Type of entity
+	(notation nil)                  ; Notation of entity
 	text				; Text of entity
 	extid				; External id 
 	)
@@ -587,7 +594,7 @@ Case transformed for general names."
       (setq name (sgml-check-name t))
       (setq dest (sgml-dtd-parameters sgml-dtd-info)))
      (t					; normal entity declaration
-      (or (sgml-parse-rni "default")
+      (or (sgml-parse-rni "DEFAULT")
 	  (setq name (sgml-check-name t)))
       (setq dest (sgml-dtd-entities sgml-dtd-info))))
     (sgml-skip-ps)
@@ -602,32 +609,35 @@ Case transformed for general names."
 					; 73 external identifier,
 					; (65 ps+, 109+ entity type)?
 	    (sgml-skip-ps)
-	    (setq type (or (sgml-parse-entity-type) 'text))
+	    (let ((tn (sgml-parse-entity-type)))
+	      (setq type (or (car tn) 'text))
+	      (unless (eq (cdr tn) "")
+		(setq notation (cdr tn))))
 	    extid)
 	   ((sgml-startnm-char-next)
-	    (let ((token (intern (sgml-check-name))))
+	    (let ((token (intern (sgml-check-case (sgml-check-name)))))
 	      (sgml-skip-ps)
 	      (when (and sgml-xml-p
 			 (memq token '(cdata sdata pi starttag endtag ms md)))
 		(sgml-error "XML forbids %s entities."
 			    (upcase (symbol-name token))))
 	      (cond
-	       ((memq token '(cdata sdata)) ; data text ***
+	       ((memq token '(CDATA SDATA)) ; data text ***
 		(setq type token)
 		(sgml-check-parameter-literal))
-	       ((eq token 'pi)
+	       ((eq token 'PI)
 		(concat "<?" (sgml-check-parameter-literal) ">"))
-	       ((eq token 'starttag)
+	       ((eq token 'STARTTAG)
 		(sgml-start-tag-of (sgml-check-parameter-literal)))
-	       ((eq token 'endtag)
+	       ((eq token 'ENDTAG)
 		(sgml-end-tag-of (sgml-check-parameter-literal)))	
-	       ((eq token 'ms)		; marked section
+	       ((eq token 'MS)		; marked section
 		(concat "<![" (sgml-check-parameter-literal) "]]>"))
-	       ((eq token 'md)		; Markup declaration
+	       ((eq token 'MD)		; Markup declaration
 		(concat "<!" (sgml-check-parameter-literal) ">")))))
 	   ((sgml-check-parameter-literal))))
     (when dest
-      (sgml-entity-declare name dest type text))))
+      (sgml-entity-declare name dest type text notation))))
 
 
 (defun sgml-parse-entity-type ()
@@ -636,16 +646,19 @@ Case transformed for general names."
   ;;                             65 ps+,
   ;;                             41 notation name,
   ;;                             149.2+ data attribute specification?)
-  (let ((type (sgml-parse-name)))
+  (let ((type (sgml-parse-name))
+	(notation nil))
     (when type
-      (setq type (intern (sgml-check-case type)))
+      (setq type (intern (downcase (sgml-check-case type))))
       (when (and sgml-xml-p (memq type '(subdoc cdata sdata)))
 	(sgml-error "XML forbids %s entities."
 		    (upcase (symbol-name type))))
       (cond ((eq type 'subdoc))
 	    ((memq type '(cdata ndata sdata))
 	     (sgml-skip-ps)
-	     (sgml-check-name)
+	     (setq notation (sgml-parse-name))
+	     (when notation
+	       (setq notation (intern (downcase (sgml-check-case notation)))))
 	     ;;149.2+ data attribute specification
 	     ;;                      = 65 ps+, DSO,
 	     ;;                        31 attribute specification list,
@@ -656,19 +669,20 @@ Case transformed for general names."
 	       (sgml-parse-s)
 	       (sgml-check-delim DSC)))
 	    (t (sgml-error "Illegal entity type: %s" type))))
-    type))
+    (cons type notation)
+    ))
 
 
 ;;;; Parse doctype: Attlist
 
 (defun sgml-declare-attlist ()
-  (let* ((assnot (cond ((sgml-parse-rni "notation")
+  (let* ((assnot (cond ((sgml-parse-rni "NOTATION")
 			(when sgml-xml-p
 			  (sgml-error "XML forbids data attribute declarations"))
 			(sgml-skip-ps)
 			t)))
 	 (assel (sgml-check-name-group))
-	 (attlist nil)			; the list
+	 (attlist nil)
 	 (attdef nil))
     (when (and sgml-xml-p (> (length assel) 1))
       (sgml-error "XML forbids name groups for an associated element type."))
@@ -685,14 +699,15 @@ Case transformed for general names."
 		   attlist))))))
 
 (defun sgml-merge-attlists (old new)
+  (setq old (nreverse (copy-list old)))
   (loop for att in new do
 	(unless (assoc (car att) old)
 	  (setq old (cons att old))))
-  old)
+  (nreverse old))
 
 (defun sgml-parse-attribute-definition ()
   (sgml-skip-ps)
-  (if (sgml-is-delim MDC) ; End of attlist?
+  (if (sgml-is-delim "MDC") ; End of attlist?
       nil
     (sgml-make-attdecl (sgml-check-name)
 		       (sgml-check-declared-value)
@@ -706,30 +721,30 @@ Case transformed for general names."
       (setq type (intern (sgml-check-case (sgml-check-name))))
       (sgml-validate-declared-value type)
       (sgml-skip-ps))
-    (when (memq type '(name-token-group notation))
+    (when (memq type '(name-token-group NOTATION))
       (setq names (sgml-check-nametoken-group)))
     (sgml-make-declared-value type names)))
 
 (defun sgml-validate-declared-value (type)
   (unless (memq type
-		'(cdata
-		  entity
-		  entities
-		  id
-		  idref
-		  idrefs
-		  name
-		  names
-		  nmtoken
-		  nmtokens
-		  notation
-		  number
-		  numbers
-		  nutoken
-		  nutokens))
+		'(CDATA
+		  ENTITY
+		  ENTITIES
+		  ID
+		  IDREF
+		  IDREFS
+		  NAME
+		  NAMES
+		  NMTOKEN
+		  NMTOKENS
+		  NOTATION
+		  NUMBER
+		  NUMBERS
+		  NUTOKEN
+		  NUTOKENS))
     (sgml-error "Invalid attribute declared value: %s" type))
   (when (and sgml-xml-p (memq type
-			      '(name names number numbers nutoken nutokens)))
+			      '(NAME NAMES NUMBER NUMBERS NUTOKEN NUTOKENS)))
     (sgml-error "XML forbids %s attributes." (upcase (symbol-name type)))))
 
 (defun sgml-check-default-value ()
@@ -740,19 +755,19 @@ Case transformed for general names."
     (sgml-skip-ps)
     (sgml-make-default-value
      key
-     (if (or (not rni) (eq key 'fixed))
+     (if (or (not rni) (eq key 'FIXED))
 	 (sgml-check-attribute-value-specification)))))
 
 (defun sgml-validate-default-value-rn (rn)
-  (unless (memq rn '(required fixed current conref implied))
+  (unless (memq rn '(REQUIRED FIXED CURRENT CONREF IMPLIED))
     (sgml-error "Unknown reserved name: %s."
 		(upcase (symbol-name rn))))
-  (when (and sgml-xml-p (memq rn '(current conref)))
+  (when (and sgml-xml-p (memq rn '(CURRENT CONREF)))
     (sgml-error "XML forbids #%s attributes."
 		(upcase (symbol-name rn)))))
   
 
-
+
 ;;;; Parse doctype: Shortref
 
 ;;;150  short reference mapping declaration = MDO, "SHORTREF",
@@ -985,11 +1000,14 @@ FORMS should produce the binary coding of element in VAR."
 (defun sgml-write-dtd (dtd file)
   "Save the parsed dtd on FILE.
 Construct the binary coded DTD (bdtd) in the current buffer."
+  (when (fboundp 'set-buffer-multibyte)
+    (setq buffer-file-coding-system 'no-conversion)
+    (set-buffer-multibyte nil))
   (insert
    ";;; This file was created by psgml on " (current-time-string) "\n"
-   "(sgml-saved-dtd-version 6)\n")
+   "(sgml-saved-dtd-version 7)\n")
   (sgml-code-dtd dtd)
-  (setq file-type 1)
+  (set 'file-type 1)
   (write-region (point-min) (point-max) file))
 
 
