@@ -43,6 +43,11 @@
   (require 'iso-sgml))
 (require 'tempo)			;essential part of html-helper-mode
 
+(eval-when-compile
+  (require 'browse-url)
+  (require 'font-lock)
+  (require 'imenu))
+
 ;;{{{ user variables
 
 (defgroup html nil
@@ -220,7 +225,7 @@ More specifically:
 	sgml-inhibit-indent-tags     '("pre")
 	sgml-minimize-attributes     nil
 	sgml-omittag                 t
-	sgml-shortag                 t)
+	sgml-shorttag                t)
 
   ;; font-lock setup for various emacsen: XEmacs, Emacs 19.29+, Emacs <19.29.
   ;; By Ulrik Dickow <dickow@nbi.dk>.  (Last update: 05-Sep-1995).
@@ -749,7 +754,6 @@ This function can be called again, it redoes the entire menu."
     "Additional expressions to highlight in HTML mode.")
 
 (put 'html-mode 'font-lock-defaults '(html-font-lock-keywords nil t))
-(put 'html3-mode 'font-lock-defaults '(html-font-lock-keywords nil t))
 
 ;;}}}
 
@@ -916,9 +920,6 @@ get those characters to appear literally in the output."
 ;;;###autoload
 (autoload 'html-mode "psgml-html" "HTML mode." t)
 
-;;;###autoload
-(autoload 'html3-mode "psgml-html" "HTML3 mode." t)
-
 (defvar sgml-html-menu
   (cons "HTML"
 	(append '(["View in Netscape" sgml-html-netscape-file
